@@ -18,7 +18,6 @@ module Hledger.Data.Journal (
   journalCanonicaliseAmounts,
   journalConvertAmountsToCost,
   journalFinalise,
-  journalSelectingDate,
   -- * Filtering
   filterJournalPostings,
   filterJournalTransactions,
@@ -50,8 +49,10 @@ module Hledger.Data.Journal (
   tests_Hledger_Data_Journal,
 )
 where
+import Control.Monad
 import Data.List
 -- import Data.Map (findWithDefault)
+import Data.Maybe
 import Data.Ord
 import Data.Time.Calendar
 import Data.Time.LocalTime
@@ -342,13 +343,6 @@ filterJournalTransactionsByAccount apats j@Journal{jtxns=ts} = j{jtxns=filter tm
 
 -}
 
--- | Convert this journal's transactions' primary date to either the
--- actual or effective date.
-journalSelectingDate :: WhichDate -> Journal -> Journal
-journalSelectingDate ActualDate j = j
-journalSelectingDate EffectiveDate j =
-    j{jtxns=map (journalTransactionWithDate EffectiveDate) $ jtxns j}
-
 -- | Apply additional account aliases (eg from the command-line) to all postings in a journal.
 journalApplyAliases :: [(AccountName,AccountName)] -> Journal -> Journal
 journalApplyAliases aliases j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
@@ -360,19 +354,79 @@ journalApplyAliases aliases j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
 -- all transactions balance, canonicalise amount formats, close any open
 -- timelog entries and so on.
 journalFinalise :: ClockTime -> LocalTime -> FilePath -> String -> JournalContext -> Journal -> Either String Journal
-journalFinalise tclock tlocal path txt ctx j@Journal{files=fs} =
-    journalBalanceTransactions $
+journalFinalise tclock tlocal path txt ctx j@Journal{files=fs} = do
+  (journalBalanceTransactions $
     journalCanonicaliseAmounts $
     journalCloseTimeLogEntries tlocal
-    j{files=(path,txt):fs, filereadtime=tclock, jContext=ctx}
+    j{files=(path,txt):fs, filereadtime=tclock, jContext=ctx})
+  >>= journalCheckBalanceAssertions
 
+-- | Check any balance assertions in the journal and return an error
+-- message if any of them fail.
+journalCheckBalanceAssertions :: Journal -> Either String Journal
+journalCheckBalanceAssertions j = do
+  let postingsByAccount = groupBy (\p1 p2 -> paccount p1 == paccount p2) $
+                          sortBy (comparing paccount) $
+                          journalPostings j
+  forM_ postingsByAccount checkBalanceAssertionsForAccount
+  Right j
+
+-- Check any balance assertions in this sequence of postings to a single account.
+checkBalanceAssertionsForAccount :: [Posting] -> Either String ()
+checkBalanceAssertionsForAccount ps
+  | null errs = Right ()
+  | otherwise = Left $ head errs
+  where
+    errs = fst $
+           foldl' checkBalanceAssertion ([],nullmixedamt) $
+           splitAssertions $
+           sortBy (comparing postingDate) ps
+
+-- Given a starting balance, accumulated errors, and a non-null sequence of
+-- postings to a single account with a balance assertion in the last:
+-- check that the final balance matches the balance assertion.
+-- If it does, return the new balance, otherwise add an error to the
+-- error list. Intended to be called from a fold.
+checkBalanceAssertion :: ([String],MixedAmount) -> [Posting] -> ([String],MixedAmount)
+checkBalanceAssertion (errs,bal) ps
+  | null ps = (errs,bal)
+  | isNothing assertion = (errs,bal)
+  |
+    -- bal' /= assertedbal  -- MixedAmount's Eq instance currently gets confused by different precisions
+    not $ isReallyZeroMixedAmount (bal' - assertedbal)
+      = (errs++[err], bal')
+  | otherwise = (errs,bal')
+  where
+    p = last ps
+    assertion = pbalanceassertion p
+    Just assertedbal = assertion
+    bal' = sum $ [bal] ++ map pamount ps
+    err = printf "Balance assertion failed for account %s on %s\n%safter\n   %s\nexpected balance is %s, actual balance was %s."
+                 (paccount p)
+                 (show $ postingDate p)
+                 (maybe "" (("In\n"++).show) $ ptransaction p)
+                 (show p)
+                 (showMixedAmount assertedbal)
+                 (showMixedAmount bal')
+
+-- Given a sequence of postings to a single account, split it into
+-- sub-sequences consisting of ordinary postings followed by a single
+-- balance-asserting posting. Postings not followed by a balance
+-- assertion are discarded.
+splitAssertions :: [Posting] -> [[Posting]]
+splitAssertions ps
+  | null rest = [[]]
+  | otherwise = (ps'++[head rest]):splitAssertions (tail rest)
+  where
+    (ps',rest) = break (isJust . pbalanceassertion) ps
+    
 -- | Fill in any missing amounts and check that all journal transactions
 -- balance, or return an error message. This is done after parsing all
 -- amounts and working out the canonical commodities, since balancing
 -- depends on display precision. Reports only the first error encountered.
 journalBalanceTransactions :: Journal -> Either String Journal
 journalBalanceTransactions j@Journal{jtxns=ts, jcommoditystyles=ss} =
-  case sequence $ map balance ts of Right ts' -> Right j{jtxns=ts'}
+  case sequence $ map balance ts of Right ts' -> Right j{jtxns=map txnTieKnot ts'}
                                     Left e    -> Left e
       where balance = balanceTransaction (Just ss)
 
@@ -518,165 +572,75 @@ Right samplejournal = journalBalanceTransactions $
          {jtxns = [
            txnTieKnot $ Transaction {
              tdate=parsedate "2008/01/01",
-             teffectivedate=Nothing,
+             tdate2=Nothing,
              tstatus=False,
              tcode="",
              tdescription="income",
              tcomment="",
              ttags=[],
-             tpostings=[
-              Posting {
-                pstatus=False,
-                paccount="assets:bank:checking",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="income:salary",
-                pamount=(missingmixedamt),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              }
-             ],
+             tpostings=
+                 ["assets:bank:checking" `post` usd 1
+                 ,"income:salary" `post` missingamt
+                 ],
              tpreceding_comment_lines=""
            }
           ,
            txnTieKnot $ Transaction {
              tdate=parsedate "2008/06/01",
-             teffectivedate=Nothing,
+             tdate2=Nothing,
              tstatus=False,
              tcode="",
              tdescription="gift",
              tcomment="",
              ttags=[],
-             tpostings=[
-              Posting {
-                pstatus=False,
-                paccount="assets:bank:checking",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="income:gifts",
-                pamount=(missingmixedamt),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              }
-             ],
+             tpostings=
+                 ["assets:bank:checking" `post` usd 1
+                 ,"income:gifts" `post` missingamt
+                 ],
              tpreceding_comment_lines=""
            }
           ,
            txnTieKnot $ Transaction {
              tdate=parsedate "2008/06/02",
-             teffectivedate=Nothing,
+             tdate2=Nothing,
              tstatus=False,
              tcode="",
              tdescription="save",
              tcomment="",
              ttags=[],
-             tpostings=[
-              Posting {
-                pstatus=False,
-                paccount="assets:bank:saving",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="assets:bank:checking",
-                pamount=(Mixed [usd (-1)]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              }
-             ],
+             tpostings=
+                 ["assets:bank:saving" `post` usd 1
+                 ,"assets:bank:checking" `post` usd (-1)
+                 ],
              tpreceding_comment_lines=""
            }
           ,
            txnTieKnot $ Transaction {
              tdate=parsedate "2008/06/03",
-             teffectivedate=Nothing,
+             tdate2=Nothing,
              tstatus=True,
              tcode="",
              tdescription="eat & shop",
              tcomment="",
              ttags=[],
-             tpostings=[
-              Posting {
-                pstatus=False,
-                paccount="expenses:food",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="expenses:supplies",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="assets:cash",
-                pamount=(missingmixedamt),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              }
-             ],
+             tpostings=["expenses:food" `post` usd 1
+                       ,"expenses:supplies" `post` usd 1
+                       ,"assets:cash" `post` missingamt
+                       ],
              tpreceding_comment_lines=""
            }
           ,
            txnTieKnot $ Transaction {
              tdate=parsedate "2008/12/31",
-             teffectivedate=Nothing,
+             tdate2=Nothing,
              tstatus=False,
              tcode="",
              tdescription="pay off",
              tcomment="",
              ttags=[],
-             tpostings=[
-              Posting {
-                pstatus=False,
-                paccount="liabilities:debts",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="assets:bank:checking",
-                pamount=(Mixed [usd (-1)]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              }
-             ],
+             tpostings=["liabilities:debts" `post` usd 1
+                       ,"assets:bank:checking" `post` usd (-1)
+                       ],
              tpreceding_comment_lines=""
            }
           ]
