@@ -22,9 +22,7 @@ module Hledger.Data.Transaction (
   isTransactionBalanced,
   -- nonzerobalanceerror,
   -- * date operations
-  transactionActualDate,
-  transactionEffectiveDate,
-  journalTransactionWithDate,
+  transactionDate2,
   -- * arithmetic
   transactionPostingBalances,
   balanceTransaction,
@@ -59,7 +57,7 @@ instance Show PeriodicTransaction where
 nulltransaction :: Transaction
 nulltransaction = Transaction {
                     tdate=nulldate,
-                    teffectivedate=Nothing, 
+                    tdate2=Nothing,
                     tstatus=False, 
                     tcode="", 
                     tdescription="", 
@@ -97,7 +95,7 @@ tests_showTransactionUnelided = [
     nulltransaction `gives` "0000/01/01\n\n"
     nulltransaction{
       tdate=parsedate "2012/05/14",
-      teffectivedate=Just $ parsedate "2012/05/15",
+      tdate2=Just $ parsedate "2012/05/15",
       tstatus=False,
       tcode="code",
       tdescription="desc",
@@ -115,14 +113,13 @@ tests_showTransactionUnelided = [
        ]
       }
       `gives` unlines [
-      "2012/05/14=2012/05/15 (code) desc  ; tcomment1",
-      "    ; tcomment2",
-      "    ; ttag1: val1",
+      "2012/05/14=2012/05/15 (code) desc",
+      "    ;tcomment1",
+      "    ;tcomment2",
       "                $1.00",
-      "    * a          2.0h  ; pcomment1",
-      "    ; pcomment2",
-      "    ; ptag1: val1",
-      "    ; ptag2: val2",
+      "    * a          2.0h",
+      "    ;pcomment1",
+      "    ;pcomment2",
       ""
       ]
  ]
@@ -131,26 +128,29 @@ tests_showTransactionUnelided = [
 showTransaction' :: Bool -> Transaction -> String
 showTransaction' elide t =
     unlines $ [descriptionline]
-              ++ commentlines
-              ++ (tagsAsLines $ ttags t)
+              ++ multilinecomment
               ++ (postingsAsLines elide t (tpostings t))
               ++ [""]
     where
-      descriptionline = rstrip $ concat [date, status, code, desc, firstcomment]
-      date = showdate (tdate t) ++ maybe "" showedate (teffectivedate t)
+      descriptionline = rstrip $ concat [date, status, code, desc, inlinecomment]
+      date = showdate (tdate t) ++ maybe "" showedate (tdate2 t)
       showdate = printf "%-10s" . showDate
       showedate = printf "=%s" . showdate
       status = if tstatus t then " *" else ""
       code = if length (tcode t) > 0 then printf " (%s)" $ tcode t else ""
       desc = if null d then "" else " " ++ d where d = tdescription t
-      (firstcomment, commentlines) = commentLines $ tcomment t
+      (inlinecomment, multilinecomment) = commentLines $ tcomment t
 
--- Render a transaction or posting's comment as indented & prefixed comment lines.
+-- Render a transaction or posting's comment as indented, semicolon-prefixed comment lines -
+-- an inline comment (when it's a single line) or multiple lines.
 commentLines :: String -> (String, [String])
 commentLines s
     | null s = ("", [])
-    | otherwise = ("  ; " ++ first, map (indent . ("; "++)) rest)
-    where (first:rest) = lines s
+    | length ls == 1 = (prefix $ head ls, [])
+    | otherwise = ("", (prefix $ head ls):(map prefix $ tail ls))
+    where
+      ls = lines s
+      prefix = indent . (";"++)
 
 postingsAsLines :: Bool -> Transaction -> [Posting] -> [String]
 postingsAsLines elide t ps
@@ -161,12 +161,11 @@ postingsAsLines elide t ps
 postingAsLines :: Bool -> [Posting] -> Posting -> [String]
 postingAsLines elideamount ps p =
     postinglines
-    ++ commentlines
-    ++ tagsAsLines (ptags p)
+    ++ multilinecomment
   where
-    postinglines = map rstrip $ lines $ concatTopPadded [showacct p, "  ", amount, firstcomment]
+    postinglines = map rstrip $ lines $ concatTopPadded [showacct p, "  ", amount, inlinecomment]
     amount = if elideamount then "" else showamt (pamount p)
-    (firstcomment, commentlines) = commentLines $ pcomment p
+    (inlinecomment, multilinecomment) = commentLines $ pcomment p
     showacct p =
       indent $ showstatus p ++ printf (printf "%%-%ds" w) (showAccountName Nothing (ptype p) (paccount p))
         where
@@ -178,22 +177,22 @@ postingAsLines elideamount ps p =
 tests_postingAsLines = [
    "postingAsLines" ~: do
     let p `gives` ls = assertEqual "" ls (postingAsLines False [p] p)
-    nullposting `gives` ["                 0"]
-    nullposting{
+    posting `gives` ["                 0"]
+    posting{
       pstatus=True,
       paccount="a",
       pamount=Mixed [usd 1, hrs 2],
-      pcomment="pcomment1\npcomment2\n",
+      pcomment="pcomment1\npcomment2\n  tag3: val3  \n",
       ptype=RegularPosting,
       ptags=[("ptag1","val1"),("ptag2","val2")]
       }
      `gives` [
       "                $1.00",
-      "    * a          2.0h  ; pcomment1",
-      "    ; pcomment2",
-      "    ; ptag1: val1",
-      "    ; ptag2: val2"
-      ]      
+      "    * a          2.0h",
+      "    ;pcomment1",
+      "    ;pcomment2",
+      "    ;  tag3: val3  "
+      ]
  ]
 
 indent :: String -> String
@@ -265,7 +264,7 @@ balanceTransaction styles t@Transaction{tpostings=ps}
     | length rwithoutamounts > 1 || length bvwithoutamounts > 1
         = Left $ printerr "could not balance this transaction (too many missing amounts)"
     | not $ isTransactionBalanced styles t''' = Left $ printerr $ nonzerobalanceerror t'''
-    | otherwise = Right t'''
+    | otherwise = Right t''''
     where
       -- maybe infer missing amounts
       (rwithamounts, rwithoutamounts)   = partition hasAmount $ realPostings t
@@ -330,6 +329,9 @@ balanceTransaction styles t@Transaction{tpostings=ps}
                         bvamountsinunpricedcommodity = filter ((==unpricedcommodity).acommodity) bvamountsinorder
             inferprice p = p
 
+      -- tie the knot so eg relatedPostings works right
+      t'''' = txnTieKnot t'''
+
       printerr s = intercalate "\n" [s, showTransactionUnelided t]
 
 nonzerobalanceerror :: Transaction -> String
@@ -342,18 +344,9 @@ nonzerobalanceerror t = printf "could not balance this transaction (%s%s%s)" rms
             | otherwise = "balanced virtual postings are off by " ++ showMixedAmount (costOfMixedAmount bvsum)
       sep = if not (null rmsg) && not (null bvmsg) then "; " else "" :: String
 
-transactionActualDate :: Transaction -> Day
-transactionActualDate = tdate
-
--- Get a transaction's effective date, defaulting to the actual date.
-transactionEffectiveDate :: Transaction -> Day
-transactionEffectiveDate t = fromMaybe (tdate t) $ teffectivedate t
-
--- | Once we no longer need both, set the main transaction date to either
--- the actual or effective date. A bit hacky.
-journalTransactionWithDate :: WhichDate -> Transaction -> Transaction
-journalTransactionWithDate ActualDate t = t
-journalTransactionWithDate EffectiveDate t = txnTieKnot t{tdate=transactionEffectiveDate t}
+-- Get a transaction's secondary date, defaulting to the primary date.
+transactionDate2 :: Transaction -> Day
+transactionDate2 t = fromMaybe (tdate t) $ tdate2 t
 
 -- | Ensure a transaction's postings refer back to it.
 txnTieKnot :: Transaction -> Transaction
@@ -376,8 +369,8 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (let t = Transaction (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
-                [Posting False "expenses:food:groceries" (Mixed [usd 47.18]) "" RegularPosting [] (Just t)
-                ,Posting False "assets:checking" (Mixed [usd (-47.18)]) "" RegularPosting [] (Just t)
+                [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
+                ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
         in showTransaction t)
 
@@ -390,8 +383,8 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (let t = Transaction (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
-                [Posting False "expenses:food:groceries" (Mixed [usd 47.18]) "" RegularPosting [] (Just t)
-                ,Posting False "assets:checking" (Mixed [usd (-47.18)]) "" RegularPosting [] (Just t)
+                [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
+                ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
         in showTransactionUnelided t)
 
@@ -406,8 +399,8 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ])
        (showTransaction
         (txnTieKnot $ Transaction (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
-         [Posting False "expenses:food:groceries" (Mixed [usd 47.18]) "" RegularPosting [] Nothing
-         ,Posting False "assets:checking" (Mixed [usd (-47.19)]) "" RegularPosting [] Nothing
+         [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
+         ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.19)]}
          ] ""))
 
   ,"showTransaction" ~: do
@@ -419,7 +412,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ])
        (showTransaction
         (txnTieKnot $ Transaction (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
-         [Posting False "expenses:food:groceries" (Mixed [usd 47.18]) "" RegularPosting [] Nothing
+         [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
          ] ""))
 
   ,"showTransaction" ~: do
@@ -431,7 +424,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ])
        (showTransaction
         (txnTieKnot $ Transaction (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
-         [Posting False "expenses:food:groceries" missingmixedamt "" RegularPosting [] Nothing
+         [posting{paccount="expenses:food:groceries", pamount=missingmixedamt}
          ] ""))
 
   ,"showTransaction" ~: do
@@ -444,28 +437,28 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ])
        (showTransaction
         (txnTieKnot $ Transaction (parsedate "2010/01/01") Nothing False "" "x" "" []
-         [Posting False "a" (Mixed [num 1 `at` (usd 2 `withPrecision` 0)]) "" RegularPosting [] Nothing
-         ,Posting False "b" missingmixedamt "" RegularPosting [] Nothing
+         [posting{paccount="a", pamount=Mixed [num 1 `at` (usd 2 `withPrecision` 0)]}
+         ,posting{paccount="b", pamount= missingmixedamt}
          ] ""))
 
   ,"balanceTransaction" ~: do
      assertBool "detect unbalanced entry, sign error"
                     (isLeft $ balanceTransaction Nothing
                            (Transaction (parsedate "2007/01/28") Nothing False "" "test" "" []
-                            [Posting False "a" (Mixed [usd 1]) "" RegularPosting [] Nothing,
-                             Posting False "b" (Mixed [usd 1]) "" RegularPosting [] Nothing
+                            [posting{paccount="a", pamount=Mixed [usd 1]}
+                            ,posting{paccount="b", pamount=Mixed [usd 1]}
                             ] ""))
 
      assertBool "detect unbalanced entry, multiple missing amounts"
                     (isLeft $ balanceTransaction Nothing
                            (Transaction (parsedate "2007/01/28") Nothing False "" "test" "" []
-                            [Posting False "a" missingmixedamt "" RegularPosting [] Nothing,
-                             Posting False "b" missingmixedamt "" RegularPosting [] Nothing
+                            [posting{paccount="a", pamount=missingmixedamt}
+                            ,posting{paccount="b", pamount=missingmixedamt}
                             ] ""))
 
      let e = balanceTransaction Nothing (Transaction (parsedate "2007/01/28") Nothing False "" "" "" []
-                           [Posting False "a" (Mixed [usd 1]) "" RegularPosting [] Nothing,
-                            Posting False "b" missingmixedamt "" RegularPosting [] Nothing
+                           [posting{paccount="a", pamount=Mixed [usd 1]}
+                           ,posting{paccount="b", pamount=missingmixedamt}
                            ] "")
      assertBool "balanceTransaction allows one missing amount" (isRight e)
      assertEqual "balancing amount is inferred"
@@ -475,8 +468,8 @@ tests_Hledger_Data_Transaction = TestList $ concat [
                         Left _ -> error' "should not happen")
 
      let e = balanceTransaction Nothing (Transaction (parsedate "2011/01/01") Nothing False "" "" "" []
-                           [Posting False "a" (Mixed [usd 1.35]) "" RegularPosting [] Nothing,
-                            Posting False "b" (Mixed [eur (-1)]) "" RegularPosting [] Nothing
+                           [posting{paccount="a", pamount=Mixed [usd 1.35]}
+                           ,posting{paccount="b", pamount=Mixed [eur (-1)]}
                            ] "")
      assertBool "balanceTransaction can infer conversion price" (isRight e)
      assertEqual "balancing conversion price is inferred"
@@ -487,52 +480,52 @@ tests_Hledger_Data_Transaction = TestList $ concat [
 
      assertBool "balanceTransaction balances based on cost if there are unit prices" (isRight $
        balanceTransaction Nothing (Transaction (parsedate "2011/01/01") Nothing False "" "" "" []
-                           [Posting False "a" (Mixed [usd 1 `at` eur 2]) "" RegularPosting [] Nothing
-                           ,Posting False "a" (Mixed [usd (-2) `at` eur 1]) "" RegularPosting [] Nothing
+                           [posting{paccount="a", pamount=Mixed [usd 1 `at` eur 2]}
+                           ,posting{paccount="a", pamount=Mixed [usd (-2) `at` eur 1]}
                            ] ""))
 
      assertBool "balanceTransaction balances based on cost if there are total prices" (isRight $
        balanceTransaction Nothing (Transaction (parsedate "2011/01/01") Nothing False "" "" "" []
-                           [Posting False "a" (Mixed [usd 1    @@ eur 1]) "" RegularPosting [] Nothing
-                           ,Posting False "a" (Mixed [usd (-2) @@ eur 1]) "" RegularPosting [] Nothing
+                           [posting{paccount="a", pamount=Mixed [usd 1    @@ eur 1]}
+                           ,posting{paccount="a", pamount=Mixed [usd (-2) @@ eur 1]}
                            ] ""))
 
   ,"isTransactionBalanced" ~: do
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 1.00]) "" RegularPosting [] (Just t)
-             ,Posting False "c" (Mixed [usd (-1.00)]) "" RegularPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ] ""
      assertBool "detect balanced" (isTransactionBalanced Nothing t)
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 1.00]) "" RegularPosting [] (Just t)
-             ,Posting False "c" (Mixed [usd (-1.01)]) "" RegularPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.01)], ptransaction=Just t}
              ] ""
      assertBool "detect unbalanced" (not $ isTransactionBalanced Nothing t)
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 1.00]) "" RegularPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ] ""
      assertBool "detect unbalanced, one posting" (not $ isTransactionBalanced Nothing t)
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 0]) "" RegularPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 0], ptransaction=Just t}
              ] ""
      assertBool "one zero posting is considered balanced for now" (isTransactionBalanced Nothing t)
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 1.00]) "" RegularPosting [] (Just t)
-             ,Posting False "c" (Mixed [usd (-1.00)]) "" RegularPosting [] (Just t)
-             ,Posting False "d" (Mixed [usd 100]) "" VirtualPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
+             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=VirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "virtual postings don't need to balance" (isTransactionBalanced Nothing t)
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 1.00]) "" RegularPosting [] (Just t)
-             ,Posting False "c" (Mixed [usd (-1.00)]) "" RegularPosting [] (Just t)
-             ,Posting False "d" (Mixed [usd 100]) "" BalancedVirtualPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
+             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "balanced virtual postings need to balance among themselves" (not $ isTransactionBalanced Nothing t)
      let t = Transaction (parsedate "2009/01/01") Nothing False "" "a" "" []
-             [Posting False "b" (Mixed [usd 1.00]) "" RegularPosting [] (Just t)
-             ,Posting False "c" (Mixed [usd (-1.00)]) "" RegularPosting [] (Just t)
-             ,Posting False "d" (Mixed [usd 100]) "" BalancedVirtualPosting [] (Just t)
-             ,Posting False "e" (Mixed [usd (-100)]) "" BalancedVirtualPosting [] (Just t)
+             [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
+             ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
+             ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
+             ,posting{paccount="3", pamount=Mixed [usd (-100)], ptype=BalancedVirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "balanced virtual postings need to balance among themselves (2)" (isTransactionBalanced Nothing t)
 
