@@ -29,7 +29,7 @@ import System.Time (ClockTime)
 
 type SmartDate = (String,String,String)
 
-data WhichDate = ActualDate | EffectiveDate deriving (Eq,Show)
+data WhichDate = PrimaryDate | SecondaryDate deriving (Eq,Show)
 
 data DateSpan = DateSpan (Maybe Day) (Maybe Day) deriving (Eq,Show,Ord)
 
@@ -76,24 +76,27 @@ data PostingType = RegularPosting | VirtualPosting | BalancedVirtualPosting
 type Tag = (String, String)
 
 data Posting = Posting {
+      pdate :: Maybe Day,  -- ^ this posting's date, if different from the transaction's
+      pdate2 :: Maybe Day,  -- ^ this posting's secondary date, if different from the transaction's
       pstatus :: Bool,
       paccount :: AccountName,
       pamount :: MixedAmount,
       pcomment :: String, -- ^ this posting's non-tag comment lines, as a single non-indented string
       ptype :: PostingType,
       ptags :: [Tag],
-      ptransaction :: Maybe Transaction  -- ^ this posting's parent transaction (co-recursive types).
-                                         -- Tying this knot gets tedious, Maybe makes it easier/optional.
+      pbalanceassertion :: Maybe MixedAmount,  -- ^ optional: the expected balance in the account after this posting
+      ptransaction :: Maybe Transaction    -- ^ this posting's parent transaction (co-recursive types).
+                                           -- Tying this knot gets tedious, Maybe makes it easier/optional.
     }
 
 -- The equality test for postings ignores the parent transaction's
 -- identity, to avoid infinite loops.
 instance Eq Posting where
-    (==) (Posting a1 b1 c1 d1 e1 f1 _) (Posting a2 b2 c2 d2 e2 f2 _) =  a1==a2 && b1==b2 && c1==c2 && d1==d2 && e1==e2 && f1==f2
+    (==) (Posting a1 b1 c1 d1 e1 f1 g1 h1 i1 _) (Posting a2 b2 c2 d2 e2 f2 g2 h2 i2 _) =  a1==a2 && b1==b2 && c1==c2 && d1==d2 && e1==e2 && f1==f2 && g1==g2 && h1==h2 && i1==i2
 
 data Transaction = Transaction {
       tdate :: Day,
-      teffectivedate :: Maybe Day,
+      tdate2 :: Maybe Day,
       tstatus :: Bool,  -- XXX tcleared ?
       tcode :: String,
       tdescription :: String,
@@ -178,42 +181,6 @@ data Reader = Reader {
 
 instance Show Reader where show r = "Reader for "++rFormat r
 
--- data format parse/conversion rules
-
--- currently the only parse (conversion) rules are those for the CSV format
-type ParseRules = CsvRules
-
--- XXX copied from Convert.hs
-{- |
-A set of data definitions and account-matching patterns sufficient to
-convert a particular CSV data file into meaningful journal transactions. See above.
--}
-data CsvRules = CsvRules {
-      dateField :: Maybe FieldPosition,
-      dateFormat :: Maybe String,
-      statusField :: Maybe FieldPosition,
-      codeField :: Maybe FieldPosition,
-      descriptionField :: [FormatString],
-      amountField :: Maybe FieldPosition,
-      amountInField :: Maybe FieldPosition,
-      amountOutField :: Maybe FieldPosition,
-      currencyField :: Maybe FieldPosition,
-      baseCurrency :: Maybe String,
-      accountField :: Maybe FieldPosition,
-      account2Field :: Maybe FieldPosition,
-      effectiveDateField :: Maybe FieldPosition,
-      baseAccount :: AccountName,
-      accountRules :: [AccountRule],
-      skipLines :: Int
-} deriving (Show, Eq)
-
-type FieldPosition = Int
-
-type AccountRule = (
-   [(String, Maybe String)] -- list of regex match patterns with optional replacements
-  ,AccountName              -- account name to use for a transaction matching this rule
-  )
-
 -- format strings
 
 data HledgerFormatField =
@@ -247,6 +214,8 @@ data Account = Account {
   aboring :: Bool           -- ^ used in the accounts report to label elidable parents
   }
 
+
+
 -- | A Ledger has the journal it derives from, and the accounts
 -- derived from that. Accounts are accessible both list-wise and
 -- tree-wise, since each one knows its parent and subs; the first
