@@ -43,6 +43,7 @@ import Text.ParserCombinators.Parsec
 import Hledger.Utils
 import Hledger.Data.Types
 import Hledger.Data.AccountName
+import Hledger.Data.Amount (nullamt)
 import Hledger.Data.Dates
 import Hledger.Data.Posting
 import Hledger.Data.Transaction
@@ -55,12 +56,14 @@ data Query = Any              -- ^ always match
            | Not Query        -- ^ negate this match
            | Or [Query]       -- ^ match if any of these match
            | And [Query]      -- ^ match if all of these match
+           | Code String      -- ^ match if code matches this regexp
            | Desc String      -- ^ match if description matches this regexp
            | Acct String      -- ^ match postings whose account matches this regexp
-           | Date DateSpan    -- ^ match if actual date in this date span
-           | EDate DateSpan   -- ^ match if effective date in this date span
+           | Date DateSpan    -- ^ match if primary date in this date span
+           | Date2 DateSpan   -- ^ match if secondary date in this date span
            | Status Bool      -- ^ match if cleared status has this value
            | Real Bool        -- ^ match if "realness" (involves a real non-virtual account ?) has this value
+           | Amt Ordering Quantity   -- ^ match if the amount's numeric quantity is less than/greater than/equal to some value
            | Empty Bool       -- ^ if true, show zero-amount postings/accounts which are usually not shown
                               --   more of a query option than a query criteria ?
            | Depth Int        -- ^ match if account depth is less than or equal to this value
@@ -72,7 +75,7 @@ data Query = Any              -- ^ always match
 data QueryOpt = QueryOptInAcctOnly AccountName  -- ^ show an account register focussed on this account
               | QueryOptInAcct AccountName      -- ^ as above but include sub-accounts in the account register
            -- | QueryOptCostBasis      -- ^ show amounts converted to cost where possible
-           -- | QueryOptEffectiveDate  -- ^ show effective dates instead of actual dates
+           -- | QueryOptDate2  -- ^ show secondary dates instead of primary dates
     deriving (Show, Eq)
 
 -- parsing
@@ -171,6 +174,7 @@ tests_words'' = [
 prefixes = map (++":") [
      "inacctonly"
     ,"inacct"
+    ,"code"
     ,"desc"
     ,"acct"
     ,"date"
@@ -201,6 +205,7 @@ parseQueryTerm _ ('i':'n':'a':'c':'c':'t':':':s) = Right $ QueryOptInAcct s
 parseQueryTerm d ('n':'o':'t':':':s) = case parseQueryTerm d s of
                                        Left m  -> Left $ Not m
                                        Right _ -> Left Any -- not:somequeryoption will be ignored
+parseQueryTerm _ ('c':'o':'d':'e':':':s) = Left $ Code s
 parseQueryTerm _ ('d':'e':'s':'c':':':s) = Left $ Desc s
 parseQueryTerm _ ('a':'c':'c':'t':':':s) = Left $ Acct s
 parseQueryTerm d ('d':'a':'t':'e':':':s) =
@@ -208,9 +213,10 @@ parseQueryTerm d ('d':'a':'t':'e':':':s) =
                                     Right (_,span) -> Left $ Date span
 parseQueryTerm d ('e':'d':'a':'t':'e':':':s) =
         case parsePeriodExpr d s of Left _ -> Left None -- XXX should warn
-                                    Right (_,span) -> Left $ EDate span
+                                    Right (_,span) -> Left $ Date2 span
 parseQueryTerm _ ('s':'t':'a':'t':'u':'s':':':s) = Left $ Status $ parseStatus s
 parseQueryTerm _ ('r':'e':'a':'l':':':s) = Left $ Real $ parseBool s
+parseQueryTerm _ ('a':'m':'t':':':s) = Left $ Amt op q where (op, q) = parseAmountTest s
 parseQueryTerm _ ('e':'m':'p':'t':'y':':':s) = Left $ Empty $ parseBool s
 parseQueryTerm _ ('d':'e':'p':'t':'h':':':s) = Left $ Depth $ readDef 0 s
 parseQueryTerm _ ('t':'a':'g':':':s) = Left $ Tag n v where (n,v) = parseTag s
@@ -232,8 +238,30 @@ tests_parseQueryTerm = [
     "inacct:a" `gives` (Right $ QueryOptInAcct "a")
     "tag:a" `gives` (Left $ Tag "a" Nothing)
     "tag:a=some value" `gives` (Left $ Tag "a" (Just "some value"))
+    -- "amt:<0" `gives` (Left $ Amt LT 0)
+    -- "amt:=.23" `gives` (Left $ Amt EQ 0.23)
+    -- "amt:>10000.10" `gives` (Left $ Amt GT 10000.1)
  ]
 
+-- can fail
+parseAmountTest :: String -> (Ordering, Quantity)
+parseAmountTest s =
+  case s of
+    ""     -> err
+    '<':s' -> (LT, readDef err s')
+    '=':s' -> (EQ, readDef err s')
+    '>':s' -> (GT, readDef err s')
+    _      -> err
+  where err = error' $ "could not parse as operator followed by numeric quantity: "++s
+
+tests_parseAmountTest = [
+  "parseAmountTest" ~: do
+    let s `gives` r = parseAmountTest s `is` r
+    "<0" `gives` (LT,0)
+    "=0.23" `gives` (EQ,0.23)
+    ">10000.10" `gives` (GT,10000.1)
+  ]
+
 parseTag :: String -> (String, Maybe String)
 parseTag s | '=' `elem` s = (n, Just $ tail v)
            | otherwise    = (s, Nothing)
@@ -337,40 +365,40 @@ queryIsAcct _ = False
 
 -- | Does this query specify a start date and nothing else (that would
 -- filter postings prior to the date) ?
--- When the flag is true, look for a starting effective date instead.
+-- When the flag is true, look for a starting secondary date instead.
 queryIsStartDateOnly :: Bool -> Query -> Bool
 queryIsStartDateOnly _ Any = False
 queryIsStartDateOnly _ None = False
-queryIsStartDateOnly effective (Or ms) = and $ map (queryIsStartDateOnly effective) ms
-queryIsStartDateOnly effective (And ms) = and $ map (queryIsStartDateOnly effective) ms
+queryIsStartDateOnly secondary (Or ms) = and $ map (queryIsStartDateOnly secondary) ms
+queryIsStartDateOnly secondary (And ms) = and $ map (queryIsStartDateOnly secondary) ms
 queryIsStartDateOnly False (Date (DateSpan (Just _) _)) = True
-queryIsStartDateOnly True (EDate (DateSpan (Just _) _)) = True
+queryIsStartDateOnly True (Date2 (DateSpan (Just _) _)) = True
 queryIsStartDateOnly _ _ = False
 
--- | What start date (or effective date) does this query specify, if any ?
+-- | What start date (or secondary date) does this query specify, if any ?
 -- For OR expressions, use the earliest of the dates. NOT is ignored.
 queryStartDate :: Bool -> Query -> Maybe Day
-queryStartDate effective (Or ms) = earliestMaybeDate $ map (queryStartDate effective) ms
-queryStartDate effective (And ms) = latestMaybeDate $ map (queryStartDate effective) ms
+queryStartDate secondary (Or ms) = earliestMaybeDate $ map (queryStartDate secondary) ms
+queryStartDate secondary (And ms) = latestMaybeDate $ map (queryStartDate secondary) ms
 queryStartDate False (Date (DateSpan (Just d) _)) = Just d
-queryStartDate True (EDate (DateSpan (Just d) _)) = Just d
+queryStartDate True (Date2 (DateSpan (Just d) _)) = Just d
 queryStartDate _ _ = Nothing
 
 queryTermDateSpan (Date span) = Just span
 queryTermDateSpan _ = Nothing
 
--- | What date span (or effective date span) does this query specify ?
+-- | What date span (or secondary date span) does this query specify ?
 -- For OR expressions, use the widest possible span. NOT is ignored.
 queryDateSpan :: Bool -> Query -> DateSpan
-queryDateSpan effective q = spansUnion $ queryDateSpans effective q
+queryDateSpan secondary q = spansUnion $ queryDateSpans secondary q
 
--- | Extract all date (or effective date) spans specified in this query.
+-- | Extract all date (or secondary date) spans specified in this query.
 -- NOT is ignored.
 queryDateSpans :: Bool -> Query -> [DateSpan]
-queryDateSpans effective (Or qs) = concatMap (queryDateSpans effective) qs
-queryDateSpans effective (And qs) = concatMap (queryDateSpans effective) qs
+queryDateSpans secondary (Or qs) = concatMap (queryDateSpans secondary) qs
+queryDateSpans secondary (And qs) = concatMap (queryDateSpans secondary) qs
 queryDateSpans False (Date span) = [span]
-queryDateSpans True (EDate span) = [span]
+queryDateSpans True (Date2 span) = [span]
 queryDateSpans _ _ = []
 
 -- | What is the earliest of these dates, where Nothing is earliest ?
@@ -457,7 +485,7 @@ tests_matchesAccount = [
     assertBool "" $ Depth 2 `matchesAccount` "a:b"
     assertBool "" $ not $ Depth 2 `matchesAccount` "a:b:c"
     assertBool "" $ Date nulldatespan `matchesAccount` "a"
-    assertBool "" $ EDate nulldatespan `matchesAccount` "a"
+    assertBool "" $ Date2 nulldatespan `matchesAccount` "a"
     assertBool "" $ not $ (Tag "a" Nothing) `matchesAccount` "a"
  ]
 
@@ -468,18 +496,15 @@ matchesPosting (Any) _ = True
 matchesPosting (None) _ = False
 matchesPosting (Or qs) p = any (`matchesPosting` p) qs
 matchesPosting (And qs) p = all (`matchesPosting` p) qs
+matchesPosting (Code r) p = regexMatchesCI r $ maybe "" tcode $ ptransaction p
 matchesPosting (Desc r) p = regexMatchesCI r $ maybe "" tdescription $ ptransaction p
 matchesPosting (Acct r) p = regexMatchesCI r $ paccount p
-matchesPosting (Date span) p =
-    case d of Just d'  -> spanContainsDate span d'
-              Nothing -> False
-    where d = maybe Nothing (Just . tdate) $ ptransaction p
-matchesPosting (EDate span) p =
-    case postingEffectiveDate p of Just d  -> spanContainsDate span d
-                                   Nothing -> False
+matchesPosting (Date span) p = span `spanContainsDate` postingDate p
+matchesPosting (Date2 span) p = span `spanContainsDate` postingDate2 p
 matchesPosting (Status v) p = v == postingCleared p
 matchesPosting (Real v) p = v == isReal p
 matchesPosting (Depth d) Posting{paccount=a} = Depth d `matchesAccount` a
+matchesPosting (Amt op n) Posting{pamount=a} = compareMixedAmount op n a
 -- matchesPosting (Empty v) Posting{pamount=a} = v == isZeroMixedAmount a
 -- matchesPosting (Empty False) Posting{pamount=a} = True
 -- matchesPosting (Empty True) Posting{pamount=a} = isZeroMixedAmount a
@@ -488,6 +513,14 @@ matchesPosting (Tag n Nothing) p = isJust $ lookupTagByName n $ postingAllTags p
 matchesPosting (Tag n (Just v)) p = isJust $ lookupTagByNameAndValue (n,v) $ postingAllTags p
 -- matchesPosting _ _ = False
 
+-- | Is this simple mixed amount's quantity less than, equal to, or greater than this number ?
+-- For complext mixed amounts (with multiple commodities), this is always true.
+compareMixedAmount :: Ordering -> Quantity -> MixedAmount -> Bool
+compareMixedAmount op q (Mixed [])  = compareMixedAmount op q (Mixed [nullamt])
+-- compareMixedAmount op q (Mixed [a]) = strace (compare (strace $ aquantity a) (strace q)) == op
+compareMixedAmount op q (Mixed [a]) = compare (aquantity a) q == op
+compareMixedAmount _ _ _            = True
+
 tests_matchesPosting = [
    "matchesPosting" ~: do
     -- matching posting status..
@@ -523,12 +556,14 @@ matchesTransaction (Any) _ = True
 matchesTransaction (None) _ = False
 matchesTransaction (Or qs) t = any (`matchesTransaction` t) qs
 matchesTransaction (And qs) t = all (`matchesTransaction` t) qs
+matchesTransaction (Code r) t = regexMatchesCI r $ tcode t
 matchesTransaction (Desc r) t = regexMatchesCI r $ tdescription t
 matchesTransaction q@(Acct _) t = any (q `matchesPosting`) $ tpostings t
 matchesTransaction (Date span) t = spanContainsDate span $ tdate t
-matchesTransaction (EDate span) t = spanContainsDate span $ transactionEffectiveDate t
+matchesTransaction (Date2 span) t = spanContainsDate span $ transactionDate2 t
 matchesTransaction (Status v) t = v == tstatus t
 matchesTransaction (Real v) t = v == hasRealPostings t
+matchesTransaction q@(Amt _ _) t = any (q `matchesPosting`) $ tpostings t
 matchesTransaction (Empty _) _ = True
 matchesTransaction (Depth d) t = any (Depth d `matchesPosting`) $ tpostings t
 matchesTransaction (Tag n Nothing) t = isJust $ lookupTagByName n $ transactionAllTags t
@@ -560,9 +595,6 @@ matchTagName pat name = pat == name
 matchTagValue :: String -> String -> Bool
 matchTagValue pat value = regexMatchesCI pat value
 
-postingEffectiveDate :: Posting -> Maybe Day
-postingEffectiveDate p = maybe Nothing (Just . transactionEffectiveDate) $ ptransaction p
-
 -- tests
 
 tests_Hledger_Query :: Test
@@ -571,6 +603,7 @@ tests_Hledger_Query = TestList $
  ++ tests_words''
  ++ tests_filterQuery
  ++ tests_parseQueryTerm
+ ++ tests_parseAmountTest
  ++ tests_parseQuery
  ++ tests_matchesAccount
  ++ tests_matchesPosting
