@@ -18,7 +18,6 @@ module Hledger.Reports (
   intervalFromOpts,
   clearedValueFromOpts,
   whichDateFromOpts,
-  journalSelectingDateFromOpts,
   journalSelectingAmountFromOpts,
   queryFromOpts,
   queryOptsFromOpts,
@@ -36,12 +35,16 @@ module Hledger.Reports (
   TransactionsReportItem,
   triDate,
   triBalance,
+  triSimpleBalance,
+  transactionsReportByCommodity,
   journalTransactionsReport,
   accountTransactionsReport,
   -- * Accounts report
   AccountsReport,
   AccountsReportItem,
   accountsReport,
+  -- * Other "reports"
+  accountBalanceHistory,
   -- * Tests
   tests_Hledger_Reports
 )
@@ -77,7 +80,7 @@ data ReportOpts = ReportOpts {
     ,cost_           :: Bool
     ,depth_          :: Maybe Int
     ,display_        :: Maybe DisplayExp
-    ,effective_      :: Bool
+    ,date2_      :: Bool
     ,empty_          :: Bool
     ,no_elide_       :: Bool
     ,real_           :: Bool
@@ -90,6 +93,7 @@ data ReportOpts = ReportOpts {
     ,quarterly_      :: Bool
     ,yearly_         :: Bool
     ,format_         :: Maybe FormatStr
+    ,related_        :: Bool
     ,query_          :: String -- all arguments, as a string
  } deriving (Show)
 
@@ -119,6 +123,7 @@ defreportopts = ReportOpts
     def
     def
     def
+    def
 
 instance Default ReportOpts where def = defreportopts
 
@@ -153,18 +158,18 @@ clearedValueFromOpts ReportOpts{..} | cleared_   = Just True
 -- depthFromOpts :: ReportOpts -> Int
 -- depthFromOpts opts = min (fromMaybe 99999 $ depth_ opts) (queryDepth $ queryFromOpts nulldate opts)
 
--- | Report which date we will report on based on --effective.
+-- | Report which date we will report on based on --date2.
 whichDateFromOpts :: ReportOpts -> WhichDate
-whichDateFromOpts ReportOpts{..} = if effective_ then EffectiveDate else ActualDate
+whichDateFromOpts ReportOpts{..} = if date2_ then SecondaryDate else PrimaryDate
 
--- | Select a Transaction date accessor based on --effective.
+-- | Select the Transaction date accessor based on --date2.
 transactionDateFn :: ReportOpts -> (Transaction -> Day)
-transactionDateFn ReportOpts{..} = if effective_ then transactionEffectiveDate else transactionActualDate
+transactionDateFn ReportOpts{..} = if date2_ then transactionDate2 else tdate
+
+-- | Select the Posting date accessor based on --date2.
+postingDateFn :: ReportOpts -> (Posting -> Day)
+postingDateFn ReportOpts{..} = if date2_ then postingDate2 else postingDate
 
--- | Convert this journal's transactions' primary date to either the
--- actual or effective date, as per options.
-journalSelectingDateFromOpts :: ReportOpts -> Journal -> Journal
-journalSelectingDateFromOpts opts = journalSelectingDate (whichDateFromOpts opts)
 
 -- | Convert this journal's postings' amounts to the cost basis amounts if
 -- specified by options.
@@ -178,7 +183,7 @@ queryFromOpts :: Day -> ReportOpts -> Query
 queryFromOpts d opts@ReportOpts{..} = simplifyQuery $ And $ [flagsq, argsq]
   where
     flagsq = And $
-              [Date $ dateSpanFromOpts d opts]
+              [(if date2_ then Date2 else Date) $ dateSpanFromOpts d opts]
               ++ (if real_ then [Real True] else [])
               ++ (if empty_ then [Empty True] else []) -- ?
               ++ (maybe [] ((:[]) . Status) (clearedValueFromOpts opts))
@@ -194,7 +199,7 @@ tests_queryFromOpts = [
                  (queryFromOpts nulldate defreportopts{begin_=Just (parsedate "2012/01/01")
                                                       ,query_="date:'to 2013'"
                                                       })
-  assertEqual "" (EDate $ mkdatespan "2012/01/01" "2013/01/01")
+  assertEqual "" (Date2 $ mkdatespan "2012/01/01" "2013/01/01")
                  (queryFromOpts nulldate defreportopts{query_="edate:'in 2012'"})
   assertEqual "" (Or [Acct "a a", Acct "'b"])
                  (queryFromOpts nulldate defreportopts{query_="'a a' 'b"})
@@ -246,28 +251,31 @@ tests_entriesReport = [
 type PostingsReport = (String               -- label for the running balance column XXX remove
                       ,[PostingsReportItem] -- line items, one per posting
                       )
-type PostingsReportItem = (Maybe (Day, String) -- transaction date and description if this is the first posting
-                          ,Posting             -- the posting, possibly with account name depth-clipped
-                          ,MixedAmount         -- the running total after this posting
+type PostingsReportItem = (Maybe Day    -- posting date, if this is the first posting in a transaction or if it's different from the previous posting's date
+                          ,Maybe String -- transaction description, if this is the first posting in a transaction
+                          ,Posting      -- the posting, possibly with account name depth-clipped
+                          ,MixedAmount  -- the running total after this posting
                           )
 
 -- | Select postings from the journal and add running balance and other
 -- information to make a postings report. Used by eg hledger's register command.
 postingsReport :: ReportOpts -> Query -> Journal -> PostingsReport
 postingsReport opts q j = -- trace ("q: "++show q++"\nq': "++show q') $
-                          (totallabel, postingsReportItems ps nullposting depth startbal (+))
+                          (totallabel, postingsReportItems ps nullposting wd depth startbal (+))
     where
       ps | interval == NoInterval = displayableps
          | otherwise              = summarisePostingsByInterval interval depth empty reportspan displayableps
-      j' = journalSelectingDateFromOpts opts $ journalSelectingAmountFromOpts opts j
-      -- don't do depth filtering until the end
+      j' = journalSelectingAmountFromOpts opts j
+      wd = whichDateFromOpts opts
+      -- delay depth filtering until the end
       (depth, q') = (queryDepth q, filterQuery (not . queryIsDepth) q)
-      (precedingps, displayableps, _) =   dbg "ps3" $ postingsMatchingDisplayExpr (display_ opts)
+      (precedingps, displayableps, _) =   dbg "ps4" $ postingsMatchingDisplayExpr displayexpr opts
+                                        $ dbg "ps3" $ (if related_ opts then concatMap relatedPostings else id)
                                         $ dbg "ps2" $ filter (q' `matchesPosting`)
                                         $ dbg "ps1" $ journalPostings j'
       dbg :: Show a => String -> a -> a
-      -- dbg = ltrace
       dbg = flip const
+      -- dbg = lstrace
 
       empty = queryEmpty q
       displayexpr = display_ opts  -- XXX
@@ -277,10 +285,10 @@ postingsReport opts q j = -- trace ("q: "++show q++"\nq': "++show q') $
       -- with period options and any span specified with display option.
       -- The latter is not easily available, fake it for now.
       requestedspan = periodspan `spanIntersect` displayspan
-      periodspan = queryDateSpan effectivedate q
-      effectivedate = whichDateFromOpts opts == EffectiveDate
+      periodspan = queryDateSpan secondarydate q
+      secondarydate = whichDateFromOpts opts == SecondaryDate
       displayspan = postingsDateSpan ps
-          where (_,ps,_) = postingsMatchingDisplayExpr displayexpr $ journalPostings j'
+          where (_,ps,_) = postingsMatchingDisplayExpr displayexpr opts $ journalPostings j'
       matchedspan = postingsDateSpan displayableps
       reportspan | empty     = requestedspan `orDatesFrom` journalspan
                  | otherwise = requestedspan `spanIntersect` matchedspan
@@ -290,30 +298,36 @@ totallabel = "Total"
 balancelabel = "Balance"
 
 -- | Generate postings report line items.
-postingsReportItems :: [Posting] -> Posting -> Int -> MixedAmount -> (MixedAmount -> MixedAmount -> MixedAmount) -> [PostingsReportItem]
-postingsReportItems [] _ _ _ _ = []
-postingsReportItems (p:ps) pprev d b sumfn = i:(postingsReportItems ps p d b' sumfn)
+postingsReportItems :: [Posting] -> Posting -> WhichDate -> Int -> MixedAmount -> (MixedAmount -> MixedAmount -> MixedAmount) -> [PostingsReportItem]
+postingsReportItems [] _ _ _ _ _ = []
+postingsReportItems (p:ps) pprev wd d b sumfn = i:(postingsReportItems ps p wd d b' sumfn)
     where
-      i = mkpostingsReportItem isfirst p' b'
+      i = mkpostingsReportItem showdate showdesc wd p' b'
+      showdate = isfirstintxn || isdifferentdate
+      showdesc = isfirstintxn
+      isfirstintxn = ptransaction p /= ptransaction pprev
+      isdifferentdate = case wd of PrimaryDate   -> postingDate p  /= postingDate pprev
+                                   SecondaryDate -> postingDate2 p /= postingDate2 pprev
       p' = p{paccount=clipAccountName d $ paccount p}
-      isfirst = ptransaction p /= ptransaction pprev
       b' = b `sumfn` pamount p
 
--- | Generate one postings report line item, given a flag indicating
--- whether to include transaction info, the posting, and the current
--- running balance.
-mkpostingsReportItem :: Bool -> Posting -> MixedAmount -> PostingsReportItem
-mkpostingsReportItem False p b = (Nothing, p, b)
-mkpostingsReportItem True p b = (ds, p, b)
-    where ds = case ptransaction p of Just (Transaction{tdate=da,tdescription=de}) -> Just (da,de)
-                                      Nothing -> Just (nulldate,"")
+-- | Generate one postings report line item, containing the posting,
+-- the current running balance, and optionally the posting date and/or
+-- the transaction description.
+mkpostingsReportItem :: Bool -> Bool -> WhichDate -> Posting -> MixedAmount -> PostingsReportItem
+mkpostingsReportItem showdate showdesc wd p b = (if showdate then Just date else Nothing, if showdesc then Just desc else Nothing, p, b)
+    where
+      date = case wd of PrimaryDate   -> postingDate p
+                        SecondaryDate -> postingDate2 p
+      desc = maybe "" tdescription $ ptransaction p
 
 -- | Date-sort and split a list of postings into three spans - postings matched
 -- by the given display expression, and the preceding and following postings.
-postingsMatchingDisplayExpr :: Maybe String -> [Posting] -> ([Posting],[Posting],[Posting])
-postingsMatchingDisplayExpr d ps = (before, matched, after)
+-- XXX always sorts by primary date, should sort by secondary date if expression is about that
+postingsMatchingDisplayExpr :: Maybe String -> ReportOpts -> [Posting] -> ([Posting],[Posting],[Posting])
+postingsMatchingDisplayExpr d opts ps = (before, matched, after)
     where
-      sorted = sortBy (comparing postingDate) ps
+      sorted = sortBy (comparing (postingDateFn opts)) ps
       (before, rest) = break (displayExprMatches d) sorted
       (matched, after) = span (displayExprMatches d) rest
 
@@ -423,8 +437,48 @@ type TransactionsReportItem = (Transaction -- the corresponding transaction
                               )
 
 triDate (t,_,_,_,_,_) = tdate t
-triBalance (_,_,_,_,_,Mixed a) = case a of [] -> "0"
-                                           (Amount{aquantity=q}):_ -> show q
+triAmount (_,_,_,_,a,_) = a
+triBalance (_,_,_,_,_,a) = a
+triSimpleBalance (_,_,_,_,_,Mixed a) = case a of [] -> "0"
+                                                 (Amount{aquantity=q}):_ -> show q
+
+-- Split a transactions report whose items may involve several commodities,
+-- into one or more single-commodity transactions reports.
+transactionsReportByCommodity :: TransactionsReport -> [TransactionsReport]
+transactionsReportByCommodity tr =
+  [filterTransactionsReportByCommodity c tr | c <- transactionsReportCommodities tr]
+  where
+    transactionsReportCommodities (_,items) =
+      nub $ sort $ map acommodity $ concatMap (amounts . triAmount) items
+
+-- Remove transaction report items and item amount (and running
+-- balance amount) components that don't involve the specified
+-- commodity. Other item fields such as the transaction are left unchanged.
+filterTransactionsReportByCommodity :: Commodity -> TransactionsReport -> TransactionsReport
+filterTransactionsReportByCommodity c (label,items) =
+  (label, fixTransactionsReportItemBalances $ concat [filterTransactionsReportItemByCommodity c i | i <- items])
+  where
+    filterTransactionsReportItemByCommodity c (t,t2,s,o,a,bal)
+      | c `elem` cs = [item']
+      | otherwise   = []
+      where
+        cs = map acommodity $ amounts a
+        item' = (t,t2,s,o,a',bal)
+        a' = filterMixedAmountByCommodity c a
+
+    fixTransactionsReportItemBalances [] = []
+    fixTransactionsReportItemBalances [i] = [i]
+    fixTransactionsReportItemBalances items = reverse $ i:(go startbal is)
+      where
+        i:is = reverse items
+        startbal = filterMixedAmountByCommodity c $ triBalance i
+        go _ [] = []
+        go bal ((t,t2,s,o,amt,_):is) = (t,t2,s,o,amt,bal'):go bal' is
+          where bal' = bal + amt
+
+-- | Filter out all but the specified commodity from this amount.
+filterMixedAmountByCommodity :: Commodity -> MixedAmount -> MixedAmount
+filterMixedAmountByCommodity c (Mixed as) = Mixed $ filter ((==c). acommodity) as
 
 -- | Select transactions from the whole journal for a transactions report,
 -- with no \"current\" account. The end result is similar to
@@ -459,11 +513,11 @@ accountTransactionsReport opts j m thisacctquery = (label, items)
  where
      -- transactions affecting this account, in date order
      ts = sortBy (comparing tdate) $ filter (matchesTransaction thisacctquery) $ jtxns $
-          journalSelectingDateFromOpts opts $ journalSelectingAmountFromOpts opts j
+          journalSelectingAmountFromOpts opts j
      -- starting balance: if we are filtering by a start date and nothing else,
      -- the sum of postings to this account before that date; otherwise zero.
      (startbal,label) | queryIsNull m                           = (nullmixedamt,        balancelabel)
-                      | queryIsStartDateOnly (effective_ opts) m = (sumPostings priorps, balancelabel)
+                      | queryIsStartDateOnly (date2_ opts) m = (sumPostings priorps, balancelabel)
                       | otherwise                                 = (nullmixedamt,        totallabel)
                       where
                         priorps = -- ltrace "priorps" $
@@ -472,7 +526,7 @@ accountTransactionsReport opts j m thisacctquery = (label, items)
                                            And [thisacctquery, tostartdatequery]))
                                          $ transactionsPostings ts
                         tostartdatequery = Date (DateSpan Nothing startdate)
-                        startdate = queryStartDate (effective_ opts) m
+                        startdate = queryStartDate (date2_ opts) m
      items = reverse $ accountTransactionsReportItems m (Just thisacctquery) startbal negate ts
 
 -- | Generate transactions report items from a list of transactions,
@@ -531,7 +585,7 @@ type AccountsReport = ([AccountsReportItem] -- line items, one per account
                       )
 type AccountsReportItem = (AccountName  -- full account name
                           ,AccountName  -- short account name for display (the leaf name, prefixed by any boring parents immediately above)
-                          ,Int          -- how many steps to indent this account (0-based account depth excluding boring parents)
+                          ,Int          -- how many steps to indent this account (0 with --flat, otherwise the 0-based account depth excluding boring parents)
                           ,MixedAmount) -- account balance, includes subs unless --flat is present
 
 -- | Select accounts, and get their balances at the end of the selected
@@ -539,7 +593,7 @@ type AccountsReportItem = (AccountName  -- full account name
 accountsReport :: ReportOpts -> Query -> Journal -> AccountsReport
 accountsReport opts q j = (items, total)
     where
-      l =  ledgerFromJournal q $ journalSelectingDateFromOpts opts $ journalSelectingAmountFromOpts opts j
+      l =  ledgerFromJournal q $ journalSelectingAmountFromOpts opts j
       accts = clipAccounts (queryDepth q) $ ledgerRootAccount l
       accts'
           | flat_ opts = filterzeros $ tail $ flattenAccounts accts
@@ -552,7 +606,7 @@ accountsReport opts q j = (items, total)
             markboring | no_elide_ opts = id
                        | otherwise      = markBoringParentAccounts
       items = map (accountsReportItem opts) accts'
-      total = sum [amt | (_,_,depth,amt) <- items, depth==0]
+      total = sum [amt | (a,_,indent,amt) <- items, if flat_ opts then accountNameLevel a == 1 else indent == 0]
 
 -- | In an account tree with zero-balance leaves removed, mark the
 -- elidable parent accounts (those with one subaccount and no balance
@@ -565,15 +619,29 @@ markBoringParentAccounts = tieAccountParents . mapAccounts mark
 
 accountsReportItem :: ReportOpts -> Account -> AccountsReportItem
 accountsReportItem opts a@Account{aname=name, aibalance=ibal}
-  | flat_ opts = (name, name,       0,     ibal)
-  | otherwise  = (name, elidedname, depth, ibal)
+  | flat_ opts = (name, name,       0,      ibal)
+  | otherwise  = (name, elidedname, indent, ibal)
   where
     elidedname = accountNameFromComponents (adjacentboringparentnames ++ [accountLeafName name])
     adjacentboringparentnames = reverse $ map (accountLeafName.aname) $ takeWhile aboring $ parents
-    depth = length $ filter (not.aboring) parents
+    indent = length $ filter (not.aboring) parents
     parents = init $ parentAccounts a
 
 
+-------------------------------------------------------------------------------
+
+-- | Get the historical running inclusive balance of a particular account,
+-- from earliest to latest posting date.
+-- XXX Accounts should know the Ledger & Journal they came from
+accountBalanceHistory :: ReportOpts -> Journal -> Account -> [(Day, MixedAmount)]
+accountBalanceHistory ropts j a = [(getdate t, bal) | (t,_,_,_,_,bal) <- items]
+  where
+    (_,items) = journalTransactionsReport ropts j acctquery
+    inclusivebal = True
+    acctquery = Acct $ (if inclusivebal then accountNameToAccountRegex else accountNameToAccountOnlyRegex) $ aname a
+    getdate = if date2_ ropts then transactionDate2 else tdate
+
+
 -------------------------------------------------------------------------------
 -- TESTS
 
@@ -792,7 +860,7 @@ tests_accountsReport =
      ],
      Mixed [nullamt])
 
-  ,"accountsReport with a date or effective date span" ~: do
+  ,"accountsReport with a date or secondary date span" ~: do
    (defreportopts{query_="date:'in 2009'"}, samplejournal2) `gives`
     ([],
      Mixed [nullamt])
@@ -949,32 +1017,16 @@ Right samplejournal2 = journalBalanceTransactions $
          {jtxns = [
            txnTieKnot $ Transaction {
              tdate=parsedate "2008/01/01",
-             teffectivedate=Just $ parsedate "2009/01/01",
+             tdate2=Just $ parsedate "2009/01/01",
              tstatus=False,
              tcode="",
              tdescription="income",
              tcomment="",
              ttags=[],
-             tpostings=[
-              Posting {
-                pstatus=False,
-                paccount="assets:bank:checking",
-                pamount=(Mixed [usd 1]),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              },
-              Posting {
-                pstatus=False,
-                paccount="income:salary",
-                pamount=(missingmixedamt),
-                pcomment="",
-                ptype=RegularPosting,
-                ptags=[],
-                ptransaction=Nothing
-              }
-             ],
+             tpostings=
+                 [posting {paccount="assets:bank:checking", pamount=Mixed [usd 1]}
+                 ,posting {paccount="income:salary", pamount=missingmixedamt}
+                 ],
              tpreceding_comment_lines=""
            }
           ]
