@@ -22,11 +22,12 @@ module Hledger.Utils (---- provide these frequently used modules - or not, for c
                           module Hledger.Utils,
                           Debug.Trace.trace,
                           -- module Hledger.Utils.UTF8IOCompat
-                          SystemString,fromSystemString,toSystemString,error',userError'
+                          SystemString,fromSystemString,toSystemString,error',userError',
+                          ppShow
                           -- the rest need to be done in each module I think
                           )
 where
-import Control.Monad (liftM)
+import Control.Monad (liftM, when)
 import Control.Monad.Error (MonadIO)
 import Control.Monad.IO.Class (liftIO)
 import Data.Char
@@ -39,10 +40,13 @@ import Data.Tree
 import Debug.Trace
 import System.Directory (getHomeDirectory)
 import System.FilePath((</>), isRelative)
+import System.IO
 import Test.HUnit
 import Text.ParserCombinators.Parsec
 import Text.Printf
+import Text.Regex
 import Text.RegexPR
+import Text.Show.Pretty
 -- import qualified Data.Map as Map
 -- 
 -- import Prelude hiding (readFile,writeFile,appendFile,getContents,putStr,putStrLn)
@@ -58,6 +62,8 @@ strip = lstrip . rstrip
 lstrip = dropWhile (`elem` " \t") :: String -> String
 rstrip = reverse . lstrip . reverse
 
+stripbrackets = dropWhile (`elem` "([") . reverse . dropWhile (`elem` "])") . reverse :: String -> String
+
 elideLeft width s =
     if length s > width then ".." ++ reverse (take (width - 2) $ reverse s) else s
 
@@ -85,9 +91,10 @@ escapeQuotes :: String -> String
 escapeQuotes = regexReplace "([\"'])" "\\1"
 
 -- | Quote-aware version of words - don't split on spaces which are inside quotes.
--- NB correctly handles "a'b" but not "''a''".
+-- NB correctly handles "a'b" but not "''a''". Can raise an error if parsing fails.
 words' :: String -> [String]
-words' = map stripquotes . fromparse . parsewith p
+words' "" = []
+words' s  = map stripquotes $ fromparse $ parsewith p s
     where
       p = do ss <- (quotedPattern <|> pattern) `sepBy` many1 spacenonewline
              -- eof
@@ -199,6 +206,7 @@ difforzero :: (Num a, Ord a) => a -> a -> a
 difforzero a b = maximum [(a - b), 0]
 
 -- regexps
+-- Note many of these will die on malformed regexps.
 
 -- regexMatch :: String -> String -> MatchFun Maybe
 regexMatch r s = matchRegexPR r s
@@ -226,6 +234,16 @@ regexReplaceBy r replfn s = gsubRegexPRBy r replfn s
 regexToCaseInsensitive :: String -> String
 regexToCaseInsensitive r = "(?i)"++ r
 
+regexSplit :: String -> String -> [String]
+regexSplit = splitRegexPR
+
+-- regex-compat (regex-posix) functions that perform better than regexpr.
+regexMatchesRegexCompat :: String -> String -> Bool
+regexMatchesRegexCompat r = isJust . matchRegex (mkRegex r)
+
+regexMatchesCIRegexCompat :: String -> String -> Bool
+regexMatchesCIRegexCompat r = isJust . matchRegex (mkRegexWithOpts r True False)
+
 -- lists
 
 splitAtElement :: Eq a => a -> [a] -> [[a]]
@@ -338,6 +356,31 @@ mtrace a = strace a `seq` return a
 tracewith :: (a -> String) -> a -> a
 tracewith f e = trace (f e) e
 
+-- | Parsec trace - show the current parsec position and next input,
+-- and the provided string if it's non-null.
+ptrace :: String -> GenParser Char st ()
+ptrace msg = do
+  pos <- getPosition
+  next <- take peeklength `fmap` getInput
+  let (l,c) = (sourceLine pos, sourceColumn pos)
+      s  = printf "at line %2d col %2d: %s" l c (show next) :: String
+      s' = printf ("%-"++show (peeklength+30)++"s") s ++ " " ++ msg
+  trace s' $ return ()
+  where
+    peeklength = 30
+
+debugLevel = 0
+
+-- | Print a message to the console if the global debugLevel is
+-- greater than the level we are called with.
+dbg :: Monad m => Float -> String -> m ()
+dbg  level msg = when (level <= debugLevel) $ trace msg $ return ()
+
+-- | Print a message and parsec position info to the console if the
+-- global debugLevel is greater than the level we are called with.
+-- pdbg :: GenParser m => Float -> String -> m ()
+pdbg level msg = when (level <= debugLevel) $ ptrace msg
+
 -- parsing
 
 -- | Backtracking choice, use this when alternatives share a prefix.
@@ -372,6 +415,9 @@ spacenonewline = satisfy (`elem` " \v\f\t")
 restofline :: GenParser Char st String
 restofline = anyChar `manyTill` newline
 
+eolof :: GenParser Char st ()
+eolof = (newline >> return ()) <|> eof
+
 -- time
 
 getCurrentLocalTime :: IO LocalTime
@@ -446,3 +492,9 @@ firstJust ms = case dropWhile (==Nothing) ms of
     [] -> Nothing
     (md:_) -> md
 
+-- | Read a file in universal newline mode, handling whatever newline convention it may contain.
+readFile' :: FilePath -> IO String
+readFile' name =  do
+  h <- openFile name ReadMode
+  hSetNewlineMode h universalNewlineMode
+  hGetContents h
